import React, {Component} from 'react';
import {DatatableHeader} from './components/DatatableHeader/DatatableHeader';
import DatatableBody from './components/DatatableBody/DatatableBody';
import {DatatableFooter} from './components/DatatableFooter/DatatableFooter';
import {createArray, deleteKeys, setColumnsVisible, selectColumn, sortColumnsToDisplay} from './components/DatatableContainerFunctions';
import uuidv4 from 'uuid/v4';
import sort from 'fast-sort';
import PrintProvider, { Print, NoPrint } from 'react-easy-print';
import {connect} from "react-redux";
import MomentUtils from '@date-io/moment';
import {MuiPickersUtilsProvider} from 'material-ui-pickers';
import moment from 'moment';
import 'moment/min/locales';

class DatatableCore extends Component {
  constructor(props) {
    super(props);
    let rows = (props.data !== undefined && Object.keys(props.data).length > 0 && props.data.rows !== undefined) ? DatatableCore.setO2DatatableRowUuid(props.data.rows) : [];
    this.state = {
      title: "",
      columns: (props.data !== undefined && Object.keys(props.data).length > 0 && props.data.columns !== undefined) ? setColumnsVisible(props.data.columns) : [],
      columnsToDisplay: (props.data !== undefined && Object.keys(props.data).length > 0 && props.data.columns !== undefined) ? setColumnsVisible(props.data.columns) : [],
      rows: createArray(rows),
      initialRows: createArray(rows),
      rowsCurrentPage: [],
      filteredRows: [],
      saving: false,
      endSorting: false,
      refreshing: false,
      searchTerm: '',
      checkedRows: [],
      editedRows: [],
      keyColumn: '',
      dateFormat: 'YYYY-MM-DDTHH:MM:ssZ', 
      font: (props.options.options.font !== undefined && props.options.options.font !== "" ? props.options.options.font : "Arial"),
      pagination: {
        rowsPerPage: props.options.options.pagination.rowsPerPage
      },
      sortBy: '[]',
      dimensions: {  
        datatable:{
          width: "850px"
        },
        body: {
          height: "300px"
        },
        header: {
          height: "60px"
        },
        row: {
          height: "60px"
        }
      },
      dimensionsComponent: {
        datatable: {
          width: 0
        },
        body: {
          height: 0
        },
        header: {
          height: 0
        },
        row: {
          height: 0
        },
        screen: {
          width: 0, 
          height: 0
        },
      },
      bodyRef: {},
      headerRef: {},
      userConfiguration: {
        columnsOrder: [],
        copyToClipboard: false
      },
      features: {
        editable: false,
        strippedRows: false,
        printable: false,
        downloadable: false,
        searchInput: false,
        canRefreshRows: false,
        canFilterColumns: false,
        canSaveColumnsConfiguration: false,
        selection: {
          rowSelectable: false,
          selectPageRows: false,
          selectAllRows: false,
        },
        additionalIcons: [],
        selectionIcons: [],
      }
    };
    this.childFooter = React.createRef();
    this.datatableHeaderRef = React.createRef();
    this.componentRef = React.createRef();
  }

  static getDerivedStateFromProps(props, state) {
    let updatedState = {};
    let dimensions = props.options.options.dimensions;
    let keyColumn = props.options.options.keyColumn;
    let title = props.options.options.title;
    let dateFormat = props.options.options.dateFormat;
    let userConfiguration = props.options.options.userConfiguration;
    let sortBy = state.sortBy;
    let columns = state.columns;
    let columnsToDisplay = state.columnsToDisplay;
    let filteredRows = state.filteredRows;
    let searchTerm = state.searchTerm;
    let pagination = state.pagination;
    let editedRows = state.editedRows;
    let checkedRows = state.checkedRows;
    let reseting = state.reseting;
    let refreshing = state.refreshing;
    let endSorting = state.endSorting;
    let saving = state.saving;
    let initialRows = state.initialRows;
    let rowsToUse = [];
    /* If it's not the first render and you are not refreshing the data */
    if(((editedRows.length > 0 || checkedRows.length > 0 || reseting) && !refreshing && !endSorting)){
      rowsToUse = state.rows;
      updatedState.reseting = false;
    }else if((saving || refreshing) && !endSorting){
      rowsToUse = createArray(DatatableCore.setO2DatatableRowUuid(props.data.rows));
      updatedState.initialRows = createArray(rowsToUse);
      updatedState.refreshing = false;
      updatedState.saving = false;
      updatedState.endSorting = false;
    }else if(endSorting){
      rowsToUse = DatatableCore.resetSortRows(state.rows, initialRows);
    }else{
      rowsToUse = state.rows;
    }
    
    /* If you have sorting condition */
    if(sortBy.length > 2){
      rowsToUse = DatatableCore.sortRows(rowsToUse, columnsToDisplay, sortBy);
      filteredRows = filteredRows.length > 0 ? DatatableCore.sortRows(filteredRows, columnsToDisplay, sortBy) : filteredRows;
    }
    
    updatedState.rows = rowsToUse;
    
    /* Check if dimensions have been updated by parent component */
    if(dimensions !== undefined && JSON.stringify(dimensions) !== JSON.stringify(state.dimensions)) {
      updatedState.dimensions = dimensions;
    }
    
    
    if(state.title !== title){
      updatedState.title = title;
    }
    
    /* If there is pagination option */
    if(Object.keys(pagination).length > 1){
      updatedState.rowsCurrentPage = (filteredRows.length > 0 || searchTerm !== '')
      ? DatatableCore.getRowsPage(filteredRows, pagination)
      : DatatableCore.getRowsPage(rowsToUse, pagination);
    }
    
    /* Check if the keyColumn has been updated */
    if(keyColumn !== state.keyColumn){
      updatedState.keyColumn = keyColumn;
    }
    
    if(props.options.options.features !== undefined){
      let editable = props.options.options.features.editable;
      let printable = props.options.options.features.printable;
      let downloadable = props.options.options.features.downloadable;
      let searchInput = props.options.options.features.searchInput;
      let canRefreshRows = props.options.options.features.canRefreshRows;
      let canFilterColumns = props.options.options.features.canFilterColumns;
      let canSaveColumnsConfiguration = props.options.options.features.canSaveColumnsConfiguration;
      let additionalIcons = props.options.options.features.additionalIcons;
      let selection = props.options.options.features.selection;
      let selectionIcons = props.options.options.features.selectionIcons;
      let strippedRows = props.options.options.features.strippedRows;
      
      updatedState.features = {
        strippedRows: (strippedRows !== undefined && strippedRows !== state.features.strippedRows) ? strippedRows : state.features.strippedRows,
        editable: (editable !== undefined && editable !== state.features.editable) ? editable : state.features.editable,
        printable: (printable !== undefined && printable !== state.features.printable) ? printable : state.features.printable,
        downloadable: (downloadable !== undefined && downloadable !== state.features.downloadable) ? downloadable : state.features.downloadable,
        searchInput: (searchInput !== undefined && searchInput !== state.features.searchInput) ? searchInput : state.features.searchInput,
        canRefreshRows: (canRefreshRows !== undefined && canRefreshRows !== state.features.canRefreshRows) ? canRefreshRows : state.features.canRefreshRows,
        canFilterColumns: (canFilterColumns !== undefined && canFilterColumns !== state.features.canFilterColumns) ? canFilterColumns : state.features.canFilterColumns,
        canSaveColumnsConfiguration: (canSaveColumnsConfiguration !== undefined && canSaveColumnsConfiguration !== state.features.canSaveColumnsConfiguration) ? canSaveColumnsConfiguration : state.features.canSaveColumnsConfiguration,
        additionalIcons: (additionalIcons !== undefined && additionalIcons.length > 0) ? additionalIcons : state.features.additionalIcons,
        selectionIcons: (selectionIcons !== undefined && selectionIcons.length > 0) ? selectionIcons : state.features.selectionIcons,
      }
      
      /* If parent component pass selection option  */
      if(selection !== undefined) {
        updatedState.features.selection = {
          rowSelectable : selection.rowsSelectable !== undefined ? selection.rowsSelectable : state.selection.rowSelectable,
          selectPageRows : selection.selectPageRows !== undefined ? selection.selectPageRows : state.selection.selectPageRows,
          selectAllRows : selection.selectAllRows !== undefined ? selection.selectAllRows : state.selection.selectAllRows,
        }
        
        if(state.columns.length > 0 && updatedState.features.selection.rowSelectable){
          updatedState.columnsToDisplay = selectColumn(columnsToDisplay);
          updatedState.columns = selectColumn(columns);
        }
      }else{
        updatedState.features.selection = state.features.selection;
      }
    }
    
    if(state.columns.length > 0 && userConfiguration !== undefined && state.userConfiguration.columnsOrder.length === 0 && userConfiguration.columnsOrder !== undefined &&  userConfiguration.columnsOrder.length > 0){
      if(updatedState.features.selection.rowSelectable && !userConfiguration.columnsOrder.includes("selectable")){
        userConfiguration.columnsOrder.unshift("selectable");
      }
      
      updatedState.userConfiguration = {
        columnsOrder: userConfiguration.columnsOrder,
        copyToClipboard: userConfiguration.copyToClipboard
      }
      columns.forEach(column => {
        column.o2DatatableColumnVisible = false;
      });
      userConfiguration.columnsOrder.forEach(colId => {
        columns.forEach(column => {
          if(column.id === colId){
            column.o2DatatableColumnVisible = true;
          }
        });
      });
      updatedState.columns = columns;
      updatedState.columnsToDisplay = sortColumnsToDisplay(columnsToDisplay, userConfiguration.columnsOrder)
    }
    
    if(dateFormat !== state.dateFormat){
      updatedState.dateFormat = dateFormat;
    }
    
    return updatedState;
  }

  /* Generate an array with the rows that the user will see */
  static getRowsPage = (rows, pagination) => {
    let firstIndex = (pagination.current - 1) * pagination.rowsPerPage.selected;
    let lastIndex = (firstIndex + pagination.rowsPerPage.selected) > rows.length ? rows.length : firstIndex + pagination.rowsPerPage.selected;
    let rowsCurrentPage = rows.slice(firstIndex, lastIndex);

    return rowsCurrentPage;
  }

  /* Sort the rows */
  static sortRows = (rows, columns, sortBy) => {
    let columnsIdWithNumber = columns.map(column => {
      return column.dataType === 'number' ? column.id : null;
    }).filter(el => el !== null);

    rows = rows.map(row => {
      Object.keys(row).forEach(key => {
        if(columnsIdWithNumber.includes(key)){
          row[key] = Number(row[key]);
        }
      });
      return row;
    });

    rows = sort(rows).by(JSON.parse(sortBy));

    rows = rows.map(row => {
      Object.keys(row).forEach(key => {
        if(columnsIdWithNumber.includes(key)){
          row[key] = row[key].toString();
        }
      });
      return row;
    });

    return rows;
  }

  /* Reset the rows to the default order */
  static resetSortRows = (rows, initialRows) => {
    let r = [];
    initialRows.forEach(ir => {
      r.push(rows.find(r => r.o2DatatableRowUuid === ir.o2DatatableRowUuid));
    });

    return r;
  }

  /* Set an unique uuid to each row */
  static setO2DatatableRowUuid = (rows) => {
    return rows.map(row => {
      row.o2DatatableRowUuid = row.o2DatatableRowUuid ? row.o2DatatableRowUuid : uuidv4();
      return row;
    })
  }

  /* Set the dimensions of the datatable */
  setDimensionsComponent = (dimensionsComponent) => {
    this.setState({dimensionsComponent: dimensionsComponent});
  }

  /* Set the pagination of the datatable */
  setPagination = (pagination) => {
    if(JSON.stringify(this.state.pagination) !== JSON.stringify(pagination)){
      this.setState({pagination: pagination});
    }
  }

  /* Set the rows that have been checked */
  setCheckedRows = (checkedRows) => {
    this.setState({checkedRows: checkedRows});
  }

  /* Set the bodyRef to access function of ref element */
  setBodyRef = (bodyRef) => {
    if(this.state.bodyRef !== bodyRef){
      this.setState({bodyRef: bodyRef});
    }
  }
  
  /* Set the headerRef to access function of ref element */
  setHeaderRef = (headerRef) => {
    if(this.state.headerRef !== headerRef){
      this.setState({headerRef: headerRef});
    }
  }

  /* Set the visibility of columns */
  setColumnsVisibility = (columns, userConfiguration = null) => {
    let columnsToDisplay = createArray(columns).filter(columns => columns.o2DatatableColumnVisible === true);
    this.setState({
      columns: columns,
      columnsToDisplay: columnsToDisplay,
    });
  }
  
  setPage = (number) => {    
    this.childFooter.current.setPage(number);
  }
  
  /* Set sorting parameter */
  setSortParameter = (val) => {
    this.setPage(1);
    let sortBy = this.state.sortBy.length > 0 ? JSON.parse(this.state.sortBy) : [];
    let isIn = sortBy.filter(sort => sort.desc === val || sort.asc === val).length > 0 ? true : false;
    if(!isIn){
      sortBy.push({
        asc: val
      });
    }else{
      sortBy = sortBy.map(sort => {
        if(sort["asc"] === val){
          sort["desc"] = sort["asc"];
          delete sort["asc"];
        }else if(sort["desc"] === val){
          delete sort["desc"];
        }
        return sort;
      }).filter(sort => Object.keys(sort).length > 0);
    }
    if(sortBy.length === 0){
      this.setState({endSorting: true});
    }
    this.setState({sortBy: JSON.stringify(sortBy)});
  }

  /* Return rows that match the searchTerm */
  getFilteredRows = (filteredRows, searchTerm) => {
    this.setState({
      filteredRows: filteredRows,
      searchTerm: searchTerm,
    });
  }

  /* Reset the rows that have been checked */
  resetRowsSelected = () => {
    let pagination = this.state.pagination;
    let rows = this.state.rows;
    rows = rows.map(row => {
      row.o2DatatableRowSelected = false;
      return row;
    });

    this.setState({
      rowsCurrentPage: DatatableCore.getRowsPage(rows, pagination),
      rows: rows,
      checkedRows: [],
    });
  }

  /* Set the columns to display */
  setColumnsToDisplay = (columnsToDisplay) => {
    this.setState({columnsToDisplay: columnsToDisplay});
  }

  /* Save the rows that have been edited */
  saveEditedRows = () => {
    let editedRows = this.state.editedRows;
    let rows = this.state.rows;
    rows = rows.map(row => {
      for(let editedRow of editedRows){
        if(row.o2DatatableRowUuid === editedRow.o2DatatableRowUuid){
          Object.keys(editedRow).forEach(key => {
            row[key] = editedRow[key];
          });
        }
      }
      return row;
    });

    editedRows = editedRows.map(editedRow => {
      delete editedRow.o2DatatableRowUuid;
      return editedRow;
    });
    
    this.datatableHeaderRef.current.activateRevertSave(false);
    this.props.saveEditedRows(editedRows);
    this.setState({
      editedRows: [],
      rows: rows,
      saving: true,
    });
  }

  /* Reset the rows that have been edited to their previous value */
  resetEditedRows = () => {
    let initialRows = this.state.initialRows;
    let checkedRows = this.state.checkedRows;
    let rows = createArray(initialRows);

    for(let checkedRow of checkedRows){
      for(let row of rows){
        if(checkedRow.o2DatatableRowUuid === row.o2DatatableRowUuid){
          row.o2DatatableRowSelected = true;
          Object.keys(row).forEach(key => {
              checkedRow[key] = row[key];
          });
        }
      }
    }
    
    this.datatableHeaderRef.current.activateRevertSave(false);
    this.setState({
      editedRows: [],
      rows: rows,
      reseting: true,
    });
  }

  /* Refresh the data of the datatable */
  refreshRows = () => {
    this.setState({refreshing: true});
    this.resetRowsSelected();
    this.resetEditedRows();
    this.props.refreshRows();
  }

  /* Reset the user configuration */
  resetUserConfiguration = (copyToClipboard) => {
    if(copyToClipboard !== null){
      this.setState({
        columnsToDisplay: createArray(this.state.columns),
        userConfiguration: {
          columnsOrder: [],
          copyToClipboard: copyToClipboard
        }
      });
    }else{
      this.setState({
        columnsToDisplay: createArray(this.state.columns),
        userConfiguration: {
          columnsOrder: [],
          copyToClipboard: this.state.userConfiguration.copyToClipboard
        }
      });
    }
    this.componentRef.current.resetColumnsToDisplay();
  }

  /* Save the user configuration */
  saveUserConfiguration = (userConfiguration) => {
    this.setState({userConfiguration: userConfiguration});
    this.props.saveUserConfiguration(userConfiguration);
  }

  handleAdditionalIconDispatch = (dispatchFunction) => {
    this.props.onAdditionalIconDispatch(dispatchFunction);
  }

  handleSelectionIconDispatch = (dispatchFunction, selectedKeys) => {
    this.props.onSelectionIconDispatch(dispatchFunction, selectedKeys);
  }
  
  /* Add the edited row with the new value in an array */
  editCell = (value, index, column, type, addToState) => {
    let dateFormat = this.state.dateFormat;
    let keyColumn = this.state.keyColumn;
    let rowsCurrentPage = this.state.rowsCurrentPage;
    let initialRows = this.state.initialRows;
    let editedRows = createArray(this.state.editedRows);
    let newValue = value;
    let row = rowsCurrentPage[index];
    
    switch (type) {
      case "date":{
        newValue = newValue.format(dateFormat);
        break;
      }
      case "checkbox": {
        newValue = newValue === "Y" ? "N" : "Y";
        break;
      }
      default:{
        break;
      }
    }
    
    let rowIn = editedRows.filter(editedRow => editedRow.o2DatatableRowUuid === row.o2DatatableRowUuid)[0];
    let indexRowIn = editedRows.findIndex(editedRow => editedRow.o2DatatableRowUuid === row.o2DatatableRowUuid);
    let indexInitialRow = initialRows.findIndex(r => r.o2DatatableRowUuid === row.o2DatatableRowUuid); 
    let initialRowValue = initialRows[indexInitialRow][column];
    
    if(type === "date" && initialRowValue !== null){
      newValue = newValue.substr(0, 10) === initialRowValue.substr(0, 10) ? initialRowValue : newValue;
    }
    
    if(rowIn === undefined && newValue !== initialRowValue){
      row[column] = newValue;
      editedRows.push({o2DatatableRowUuid: row.o2DatatableRowUuid, [keyColumn]: row[keyColumn], [column]: row[column]})      
    }else if(rowIn !== undefined && newValue === initialRowValue){
      let deleteKey = Object.keys(editedRows[indexRowIn]).filter(el => el !== "o2DatatableRowUuid" && el !== keyColumn).length > 1 ? true : false;
      row[column] = initialRowValue;
      if(deleteKey){
        delete editedRows[indexRowIn][column];
      }else{        
        editedRows.splice(indexRowIn, 1);
      }
    }else if(rowIn !== undefined){        
      rowIn[column] = newValue;
      row[column] = newValue;
    }
    
    let bool = editedRows.length === 0 ? false : true;
    this.datatableHeaderRef.current.activateRevertSave(bool);
    if(addToState){
      this.setState({editedRows: editedRows});
    }
  }
  
  render() {
    const {
      features, pagination, keyColumn, userConfiguration, initialRows, rowsCurrentPage, columns, headerRef, title, columnsToDisplay, dimensions, sortBy, searchTerm, editedRows, 
      dimensionsComponent, rows, filteredRows, bodyRef, checkedRows, font
    } = this.state;
    
    return (
      <MuiPickersUtilsProvider utils={MomentUtils}>
        <div className={"o2"} style={{
          width: dimensionsComponent.datatable.width + 8
        }}>
          <PrintProvider>
            <NoPrint>
              <DatatableHeader ref={this.datatableHeaderRef} columns={columns} rows={rows} editable={features.editable} checkedRows={checkedRows} 
                editedRows={editedRows} title={title} resetRowsSelected={this.resetRowsSelected} printable={features.printable} 
                downloadable={features.downloadable} searchInput={features.searchInput} canRefreshRows={features.canRefreshRows} 
                canFilterColumns={features.canFilterColumns} dimensionsComponent={dimensionsComponent} rowsCurrentPage={rowsCurrentPage}
                canSaveColumnsConfiguration={features.canSaveColumnsConfiguration} userConfiguration={userConfiguration} resetEditedRows={this.resetEditedRows}
                refreshRows={this.refreshRows} columnsToDisplay={columnsToDisplay} saveEditedRows={this.saveEditedRows} setPage={this.setPage}
                setColumnsVisibility={this.setColumnsVisibility} additionalIcons={features.additionalIcons} selectionIcons={features.selectionIcons}
                getFilteredRows={this.getFilteredRows} saveUserConfiguration={this.saveUserConfiguration} resetUserConfiguration={this.resetUserConfiguration}
                rowSelectable={features.selection.rowSelectable} handleAdditionalIconDispatch={this.handleAdditionalIconDispatch} dimensions={dimensions}
                headerRef={headerRef} bodyRef={bodyRef} handleSelectionIconDispatch={this.handleSelectionIconDispatch} keyColumn={keyColumn}
              />
              
              <Print single>
                <DatatableBody ref={this.componentRef} pagination={pagination} rows={rows} filteredRows={filteredRows} checkedRows={checkedRows}
                  editable={features.editable} initialRows={initialRows} columns={columns} editCell={this.editCell} strippedRows={features.strippedRows} selection={features.selection}
                  sortBy={sortBy} rowsCurrentPage={rowsCurrentPage} keyColumn={keyColumn} columnsToDisplay={columnsToDisplay} setBodyRef={this.setBodyRef}
                  dimensions={dimensions} searchTerm={searchTerm} setCheckedRows={this.setCheckedRows} setDimensionsComponent={this.setDimensionsComponent}
                  font={font} setColumnsToDisplay={this.setColumnsToDisplay} setSortParameter={this.setSortParameter} setEditedRows={this.setEditedRows}
                  setHeaderRef={this.setHeaderRef} userConfiguration={userConfiguration}
                />
              </Print>
              
              <DatatableFooter ref={this.childFooter} pagination={pagination} rows={rows} filteredRows={filteredRows} searchTerm={searchTerm}
                dimensionsComponent={dimensionsComponent} bodyRef={bodyRef} setPagination={this.setPagination} columns={columns}
              />
            </NoPrint>
          </PrintProvider>
        </div>
      </MuiPickersUtilsProvider>
    );
  }
}


const mapDispatchToProps = (dispatch) => {
  return {
    onAdditionalIconDispatch: (dispatchFunction) => dispatch(dispatchFunction),
    onSelectionIconDispatch: (dispatchFunction, selectedKeys) => dispatch(dispatchFunction(selectedKeys))
  };
};

class Datatable extends Component {
   render(){
     let ConnectedDatatable;
     if(!this.props.noRedux) {
       ConnectedDatatable = connect(null, mapDispatchToProps)(DatatableCore);
     } else {
       ConnectedDatatable = (DatatableCore);
     }
    return (<ConnectedDatatable data={this.props.data} options={this.props} refreshRows={this.props.refreshRows} onChange={this.props.onChange}
      getCheckedRows={this.props.getCheckedRows} saveEditedRows={this.props.saveEditedRows} saveUserConfiguration={this.props.saveUserConfiguration}/>);
 }
}

window.__MUI_USE_NEXT_TYPOGRAPHY_VARIANTS__ = true;

export {Datatable};