import React, {Component} from 'react';
import {Select, MenuItem, IconButton} from '@material-ui/core';
import {NavigateNext as NavigateNextIcon, NavigateBefore as NavigateBeforeIcon} from '@material-ui/icons';
import {scrollTo, calculateMinusMax} from './DatatableFooterFunctions';

class DatatableFooter extends Component {
  constructor(props) {
    super(props);
    this.state = {
      rows: [],
      filteredRows: [],
      columns: [],
      searchTerm: '',
      dimensionsComponent: {},
      bodyRef: {},
      pagination:{
        total: 0,
        current: 1,
        rowsPerPage: props.pagination.rowsPerPage
      }
    };
  }
  
  static getDerivedStateFromProps(props, state) {
    let updatedState = {};
    let rows = props.rows;
    let columns = props.columns;
    let filteredRows = props.filteredRows;
    let dimensionsComponent = props.dimensionsComponent;
    let bodyRef = props.bodyRef;
    let searchTerm = props.searchTerm;
    let rowsToUse = (filteredRows.length > 0 || searchTerm !== '') ? filteredRows : rows;
    
    updatedState.rows = rows;
    updatedState.columns = columns;
    updatedState.filteredRows = filteredRows;
    
    
    if(searchTerm !== state.searchTerm){
      updatedState.searchTerm = searchTerm;
    }
    
    if(JSON.stringify(state.pagination) !== JSON.stringify(props.pagination) || searchTerm !== state.searchTerm || JSON.stringify(filteredRows) !== JSON.stringify(state.filteredRows) || JSON.stringify(rows) !== JSON.stringify(state.rows)){
      let pagination = {
        total: filteredRows.length === 0 && searchTerm !== '' ? 1 : Math.ceil(rowsToUse.length/state.pagination.rowsPerPage.selected),
        current: filteredRows.length === 0 && searchTerm !== '' ? 1 : state.pagination.current,
        rowsPerPage: state.pagination.rowsPerPage
      };
      
      pagination.range = calculateMinusMax(pagination, rowsToUse);  
      updatedState.pagination = pagination;
      props.setPagination(pagination);
    }
    
    if(bodyRef !== state.bodyRef){
      updatedState.bodyRef = bodyRef;
    }

    if (dimensionsComponent !== state.dimensionsComponent) {
      updatedState.dimensionsComponent = dimensionsComponent;
    }

    return updatedState;
  }
  
  /* Change the current page */
  changePage = (event) => {
    this.setState({
      pagination:{
        total: this.state.pagination.total,
        current: event.target.value,
        rowsPerPage: this.state.pagination.rowsPerPage
      }
    });
  
    scrollTo(this.state.bodyRef._bottomRightGrid._scrollingContainer, 0, 'smooth');
  }
  
  /* Change the number of rows per page */
  changeRowsPerPage = (event, rowsPage) => {
    let selected = event.target.value;
    let scrollHeight = this.state.bodyRef.state.scrollTop;
    let rowHeight = this.state.dimensionsComponent.row.height;
    let index = Math.round(scrollHeight/rowHeight) + (this.state.pagination.rowsPerPage.selected * (this.state.pagination.current - 1));
    let current = 1;

    while(index > selected){
      current ++;
      index -= selected;
    }
    
    let rows = this.state.filteredRows.length > 0 ? this.state.filteredRows : this.state.rows;
    
    this.setState({
      pagination:{
        total: Math.ceil(rows.length/event.target.value),
        current: current,
        rowsPerPage: {
          available: this.state.pagination.rowsPerPage.available,
          selected: selected
        }
      }
    });
    
    scrollTo(this.state.bodyRef._bottomRightGrid._scrollingContainer, 0, 'instant');
    setTimeout(() => { scrollTo(this.state.bodyRef._bottomRightGrid._scrollingContainer, index * rowHeight, 'smooth') }, 400);
  }
  
  /* Navigate to next page */
  nextPage = () => {
    let pagination = this.state.pagination;
    
    this.setState({
      pagination:{
        total: pagination.total,
        current: pagination.current + 1,
        rowsPerPage: pagination.rowsPerPage
      }
    });
    
    scrollTo(this.state.bodyRef._bottomRightGrid._scrollingContainer, 0, 'smooth');
  }
  
  /* Navigate to previous page */
  previousPage = () => {
    let pagination = this.state.pagination;
    
    this.setState({
      pagination:{
        total: pagination.total,
        current: pagination.current - 1,
        rowsPerPage: pagination.rowsPerPage
      }
    });
    
    scrollTo(this.state.bodyRef._bottomRightGrid._scrollingContainer, 0, 'smooth');
  }
  
  /* Select the page */
  setPage = (pageNumber) => {
    let pagination = this.state.pagination;
    
    this.setState({
      pagination:{
        total: pagination.total,
        current: pageNumber <= pagination.total ? pageNumber : pagination.total,
        rowsPerPage: pagination.rowsPerPage
      }
    });
    
    scrollTo(this.state.bodyRef._bottomRightGrid._scrollingContainer, 0, 'smooth');
  }
  
  render() {
    const {pagination, dimensionsComponent, columns} = this.state;
    let rows = this.state.filteredRows.length > 0 || this.state.searchTerm !== '' ? this.state.filteredRows : this.state.rows;
    return (  
      <div className={"Datatable-footer"} style={{width: dimensionsComponent.body.width}}>
        <div>          
          Page: 
          <Select value={pagination.current} onChange={this.changePage} className={"Datatable-footer-select"} disabled={rows.length === 0 || columns.length === 0}>
            {Array.apply(null, {length: pagination.total}).map(Number.call, Number).map(page => {
              return (
                <MenuItem key={page+1} value={page+1}>
                  {page+1}
                </MenuItem>
              );
            })};
          </Select>
        </div>
        
        <div>          
          Rows per page: 
          <Select value={pagination.rowsPerPage.selected} onChange={this.changeRowsPerPage} className={"Datatable-footer-select"} disabled={rows.length === 0 || columns.length === 0}> 
            {pagination.rowsPerPage.available.map(nbrRow => {
              return (
                <MenuItem key={nbrRow} value={nbrRow}>
                  {nbrRow}
                </MenuItem>
              );
            })};                  
          </Select>
        </div>
        
          {(this.state.rows.length === 0 || columns.length === 0) &&
            <div>
              0-0 of 0  
            </div>  
          }
          
          {(this.state.rows.length >= 0 && columns.length > 0) &&
            <div>
              {pagination.range.first}-{pagination.range.last} of {rows.length}
            </div>
          }
        
        <div>          
          <IconButton disabled={pagination.current === 1 || rows.length === 0 || columns.length === 0} onClick={this.previousPage}>
            <NavigateBeforeIcon />
          </IconButton>
          
          <IconButton disabled={pagination.current === pagination.total || rows.length === 0 || columns.length === 0} onClick={this.nextPage}>  
            <NavigateNextIcon />
          </IconButton>
        </div>
        
      </div>
        );
      }
    }
    
    export {DatatableFooter};